@@ -1,7 +1,7 @@
 // for-of works on a cross-compartment wrapper of a Set.
 
 var g = newGlobal();
-var mw = g.eval("Set(['a', 'b', 1, 2])");
+var mw = g.eval("new Set(['a', 'b', 1, 2])");
 var log = '';
 for (let x of mw)
     log += x;
