@@ -11,14 +11,15 @@
 
 namespace js {
 
-extern void *
+extern void*
 GetNativeStackBaseImpl();
 
 inline uintptr_t
 GetNativeStackBase()
 {
     uintptr_t stackBase = reinterpret_cast<uintptr_t>(GetNativeStackBaseImpl());
-    JS_ASSERT(stackBase % sizeof(void *) == 0);
+    MOZ_ASSERT(stackBase != 0);
+    MOZ_ASSERT(stackBase % sizeof(void*) == 0);
     return stackBase;
 }
 
