@@ -4,12 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 360969;
-var summary = '2^17: local const';
+var BUGNUMBER = 456540;
+var summary = 'Do not assert with JIT: (m != JSVAL_INT) || isInt32(*vp)" with ((-1) % ""';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-var global = this;
 
 //-----------------------------------------------------------------------------
 test();
@@ -21,21 +20,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  var start = new Date();
-  var p;
-  var i;
-  var limit = 2 << 16;
+  jit(true);
 
-  for (var i = 0; i < limit; i++)
-  {
-    eval('const pv' + i + ' = undefined;');
-  }
+  for (var j = 0; j < 5; ++j) { var t = ((-1) % "" ); }
+
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
-  var stop = new Date();
-
-  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
-
   exitFunc ('test');
 }
