@@ -4,10 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 354541;
-var summary = 'Regression to standard class constructors in case labels';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 354945;
+var summary = 'Do not crash with new Iterator';
+var expect = 'TypeError: trap __iterator__ for obj returned a primitive value';
+var actual;
 
 
 //-----------------------------------------------------------------------------
@@ -18,23 +18,14 @@ function test()
 {
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
-  printStatus (summary + ': in function');
+  printStatus (summary);
 
-  String.prototype.trim = function() { print('hallo'); };
-
-  const S = String;
-  const Sp = String.prototype;
-
-  expect = 'No Error';
-  actual = 'No Error';
-  if (typeof Script == 'undefined')
-  {
-    print('Test skipped. Script not defined.');
-  }
-  else
-  {
-    var s = Script('var tmp = function(o) { switch(o) { case String: case 1: return ""; } }; print(String === S); print(String.prototype === Sp); "".trim();');
-    s();
+  try {
+    var obj = {};
+    obj.__iterator__ = function(){ };
+    for(t in (new Iterator(obj))) { }
+  } catch (ex) {
+    actual = ex.toString();
   }
 
   reportCompare(expect, actual, summary);
