@@ -4,12 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 355832;
+var summary = 'execution of let binding nothing';
 var actual = '';
 var expect = '';
 
-//-------  Comment #178  From  Gary Kwong
 
 //-----------------------------------------------------------------------------
 test();
@@ -20,10 +19,10 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  expect = actual = 'No Crash';
 
-// Assertion failure: slot < fp2->script->nfixed, at ../jsinterp.cpp:5610
-
-  eval("with({}) let(x=[])(function(){x})()");
+  let ([] = []) { }
 
   reportCompare(expect, actual, summary);
 
