@@ -4,37 +4,37 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.6/7.6.1/7.6.1-8-15.js
- * @description Allow reserved words as property names by set function within an object, accessed via indexing: package, protected, static
+ * @path ch07/7.6/7.6.1/7.6.1-8-3.js
+ * @description Allow reserved words as property names by set function within an object, accessed via indexing: instanceof, typeof, else
  */
 
 
 function testcase() {
         var test0 = 0, test1 = 1, test2 = 2;
         var tokenCodes  = {
-            set package(value){
+            set instanceof(value){
                 test0 = value;
             },
-            get package(){
+            get instanceof(){
                 return test0;
             },
-            set protected(value){
+            set typeof(value){
                 test1 = value;
             },
-            get protected(){
-                return test1
+            get typeof(){
+                return test1;
             },
-            set static(value){
+            set else(value){
                 test2 = value;
             },
-            get static(){
+            get else(){
                 return test2;
             }
         }; 
         var arr = [
-            'package',
-            'protected',
-            'static'  
+            'instanceof',
+            'typeof',
+            'else'
         ];
         for (var i = 0; i < arr.length; i++) {
             if (tokenCodes[arr[i]] !== i) {
