@@ -4,17 +4,14 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-23-s.js
- * @description Strict Mode - checking 'this' (New'ed object from FunctionExpression defined within strict mode)
+ * @path ch10/10.4/10.4.3/10.4.3-1-54-s.js
+ * @description Strict Mode - checking 'this' (Literal getter defined within strict mode)
  * @onlyStrict
  */
     
 function testcase() {
 "use strict";
-var f = function () {
-    return this;
-}
-return ( (new f())!==fnGlobalObject()) && (typeof (new f()) !== "undefined");
-
+var o = { get foo() { return this; } }
+return o.foo===o;
 }
 runTestCase(testcase);
\ No newline at end of file
