@@ -4,18 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-27gs.js
- * @description Strict - checking 'this' from a global scope (FunctionDeclaration defined within a FunctionDeclaration inside strict mode)
+ * @path ch10/10.4/10.4.3/10.4.3-1-62gs.js
+ * @description Strict - checking 'this' from a global scope (strict function declaration called by non-strict function declaration)
  * @onlyStrict
  */
 
-"use strict";
-function f1() {
-    function f() {
-        return typeof this;
-    }
-    return (f()==="undefined") && ((typeof this)==="undefined");
-}
-if (! f1()) {
+function f() { "use strict"; return this;};
+function foo() { return f();}
+if (foo()!==undefined){
     throw "'this' had incorrect value!";
 }
\ No newline at end of file
