@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -51,6 +51,7 @@ that of Atlas depending on commandline parameters.
 #include "ps/Globals.h"
 #include "ps/Hotkey.h"
 #include "ps/Loader.h"
+#include "ps/ModInstaller.h"
 #include "ps/Profile.h"
 #include "ps/Profiler2.h"
 #include "ps/Pyrogenesis.h"
@@ -83,9 +84,34 @@ that of Atlas depending on commandline parameters.
 #include <unistd.h> // geteuid
 #endif // OS_UNIX
 
-extern bool g_GameRestarted;
+#if MSC_VERSION
+#include <process.h>
+#define getpid _getpid // Use the non-deprecated function name
+#endif
 
-void kill_mainloop();
+extern CmdLineArgs g_args;
+extern CStrW g_UniqueLogPostfix;
+
+// Marks terrain as modified so the minimap can repaint (is there a cleaner way of handling this?)
+bool g_GameRestarted = false;
+
+// Determines the lifetime of the mainloop
+enum ShutdownType
+{
+	// The application shall continue the main loop.
+	None,
+
+	// The process shall terminate as soon as possible.
+	Quit,
+
+	// The engine should be restarted in the same process, for instance to activate different mods.
+	Restart,
+
+	// Atlas should be started in the same process.
+	RestartAsAtlas
+};
+
+static ShutdownType g_Shutdown = ShutdownType::None;
 
 // to avoid redundant and/or recursive resizing, we save the new
 // size after VIDEORESIZE messages and only update the video mode
@@ -97,6 +123,26 @@ static int g_ResizedH;
 
 static std::chrono::high_resolution_clock::time_point lastFrameTime;
 
+bool IsQuitRequested()
+{
+	return g_Shutdown == ShutdownType::Quit;
+}
+
+void QuitEngine()
+{
+	g_Shutdown = ShutdownType::Quit;
+}
+
+void RestartEngine()
+{
+	g_Shutdown = ShutdownType::Restart;
+}
+
+void StartAtlas()
+{
+	g_Shutdown = ShutdownType::RestartAsAtlas;
+}
+
 // main app message handler
 static InReaction MainInputHandler(const SDL_Event_* ev)
 {
@@ -121,14 +167,14 @@ static InReaction MainInputHandler(const SDL_Event_* ev)
 		break;
 
 	case SDL_QUIT:
-		kill_mainloop();
+		QuitEngine();
 		break;
 
 	case SDL_HOTKEYDOWN:
 		std::string hotkey = static_cast<const char*>(ev->ev.user.data1);
 		if (hotkey == "exit")
 		{
-			kill_mainloop();
+			QuitEngine();
 			return IN_HANDLED;
 		}
 		else if (hotkey == "screenshot")
@@ -269,9 +315,6 @@ static void RendererIncrementalLoad()
 	while (more && timer_Time() - startTime < maxTime);
 }
 
-
-static bool quit = false;	// break out of main loop
-
 static void Frame()
 {
 	g_Profiler2.RecordFrameStart();
@@ -298,8 +341,7 @@ static void Frame()
 #endif
 	ENSURE(realTimeSinceLastFrame > 0.0f);
 
-	// decide if update/render is necessary
-	bool need_render = !g_app_minimized;
+	// Decide if update is necessary
 	bool need_update = true;
 
 	// If we are not running a multiplayer game, disable updates when the game is
@@ -326,7 +368,7 @@ static void Frame()
 	// if the user quit by closing the window, the GL context will be broken and
 	// may crash when we call Render() on some drivers, so leave this loop
 	// before rendering
-	if (quit)
+	if (g_Shutdown != ShutdownType::None)
 		return;
 
 	// respond to pumped resize events
@@ -363,9 +405,11 @@ static void Frame()
 	g_UserReporter.Update();
 
 	g_Console->Update(realTimeSinceLastFrame);
-
 	ogl_WarnIfError();
-	if (need_render)
+
+	// We do not have to render an inactive fullscreen frame, because it can
+	// lead to errors for some graphic card families.
+	if (!g_app_minimized && (g_app_has_focus || !g_VideoMode.IsInFullscreen()))
 	{
 		Render();
 
@@ -396,10 +440,9 @@ static void NonVisualFrame()
 	g_Profiler.Frame();
 
 	if (g_Game->IsGameFinished())
-		kill_mainloop();
+		QuitEngine();
 }
 
-
 static void MainControllerInit()
 {
 	// add additional input handlers only needed by this controller:
@@ -408,41 +451,11 @@ static void MainControllerInit()
 	in_add_handler(MainInputHandler);
 }
 
-
-
 static void MainControllerShutdown()
 {
 	in_reset_handlers();
 }
 
-
-// stop the main loop and trigger orderly shutdown. called from several
-// places: the event handler (SDL_QUIT and hotkey) and JS exitProgram.
-void kill_mainloop()
-{
-	quit = true;
-}
-
-
-static bool restart_in_atlas = false;
-// called by game code to indicate main() should restart in Atlas mode
-// instead of terminating
-void restart_mainloop_in_atlas()
-{
-	quit = true;
-	restart_in_atlas = true;
-}
-
-static bool restart = false;
-// trigger an orderly shutdown and restart the game.
-void restart_engine()
-{
-	quit = true;
-	restart = true;
-}
-
-extern CmdLineArgs g_args;
-
 // moved into a helper function to ensure args is destroyed before
 // exit(), which may result in a memory leak.
 static void RunGameOrAtlas(int argc, const char* argv[])
@@ -451,7 +464,7 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 
 	g_args = args;
 
-	if (args.Has("version") || args.Has("-version"))
+	if (args.Has("version"))
 	{
 		debug_printf("Pyrogenesis %s\n", engine_version);
 		return;
@@ -463,6 +476,9 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 		return;
 	}
 
+	if (args.Has("unique-logs"))
+		g_UniqueLogPostfix = L"_" + std::to_wstring(std::time(nullptr)) + L"_" + std::to_wstring(getpid());
+
 	const bool isVisualReplay = args.Has("replay-visual");
 	const bool isNonVisualReplay = args.Has("replay");
 	const bool isNonVisual = args.Has("autostart-nonvisual");
@@ -485,6 +501,28 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 		}
 	}
 
+	std::vector<OsPath> modsToInstall;
+	for (const CStr& arg : args.GetArgsWithoutName())
+	{
+		const OsPath modPath(arg);
+		if (!CModInstaller::IsDefaultModExtension(modPath.Extension()))
+		{
+			debug_printf("Skipping file '%s' which does not have a mod file extension.\n", modPath.string8().c_str());
+			continue;
+		}
+		if (!FileExists(modPath))
+		{
+			debug_printf("ERROR: The mod file '%s' does not exist!\n", modPath.string8().c_str());
+			continue;
+		}
+		if (DirectoryExists(modPath))
+		{
+			debug_printf("ERROR: The mod file '%s' is a directory!\n", modPath.string8().c_str());
+			continue;
+		}
+		modsToInstall.emplace_back(std::move(modPath));
+	}
+
 	// We need to initialize SpiderMonkey and libxml2 in the main thread before
 	// any thread uses them. So initialize them here before we might run Atlas.
 	ScriptEngine scriptEngine;
@@ -506,7 +544,7 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 		}
 
 		Paths paths(args);
-		g_VFS = CreateVfs(20 * MiB);
+		g_VFS = CreateVfs();
 		g_VFS->Mount(L"cache/", paths.Cache(), VFS_MOUNT_ARCHIVABLE);
 		MountMods(paths, GetMods(args, INIT_MODS));
 
@@ -558,8 +596,8 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 	int flags = INIT_MODS;
 	do
 	{
-		restart = false;
-		quit = false;
+		g_Shutdown = ShutdownType::None;
+
 		if (!Init(args, flags))
 		{
 			flags &= ~INIT_MODS;
@@ -567,26 +605,43 @@ static void RunGameOrAtlas(int argc, const char* argv[])
 			continue;
 		}
 
+		std::vector<CStr> installedMods;
+		if (!modsToInstall.empty())
+		{
+			Paths paths(args);
+			CModInstaller installer(paths.UserData() / "mods", paths.Cache());
+
+			// Install the mods without deleting the pyromod files
+			for (const OsPath& modPath : modsToInstall)
+				installer.Install(modPath, g_ScriptRuntime, true);
+
+			installedMods = installer.GetInstalledMods();
+		}
+
 		if (isNonVisual)
 		{
 			InitNonVisual(args);
-			while (!quit)
+			while (g_Shutdown == ShutdownType::None)
 				NonVisualFrame();
 		}
 		else
 		{
-			InitGraphics(args, 0);
+			InitGraphics(args, 0, installedMods);
 			MainControllerInit();
-			while (!quit)
+			while (g_Shutdown == ShutdownType::None)
 				Frame();
 		}
 
+		// Do not install mods again in case of restart (typically from the mod selector)
+		modsToInstall.clear();
+
 		Shutdown(0);
 		MainControllerShutdown();
 		flags &= ~INIT_MODS;
-	} while (restart);
 
-	if (restart_in_atlas)
+	} while (g_Shutdown == ShutdownType::Restart);
+
+	if (g_Shutdown == ShutdownType::RestartAsAtlas)
 		ATLAS_RunIfOnCmdLine(args, true);
 
 	CXeromyces::Terminate();
