@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -26,12 +26,12 @@
 #include "lib/byte_order.h"
 #include "lib/external_libraries/enet.h"
 #include "lib/sysdep/sysdep.h"
+#include "lobby/IXmppClient.h"
 #include "ps/CConsole.h"
 #include "ps/CLogger.h"
 #include "ps/Compress.h"
 #include "ps/CStr.h"
 #include "ps/Game.h"
-#include "ps/GUID.h"
 #include "ps/Loader.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "simulation2/Simulation2.h"
@@ -70,7 +70,7 @@ private:
 CNetClient::CNetClient(CGame* game, bool isLocalClient) :
 	m_Session(NULL),
 	m_UserName(L"anonymous"),
-	m_GUID(ps_generate_guid()), m_HostID((u32)-1), m_ClientTurnManager(NULL), m_Game(game),
+	m_HostID((u32)-1), m_ClientTurnManager(NULL), m_Game(game),
 	m_GameAttributes(game->GetSimulation2()->GetScriptInterface().GetContext()),
 	m_IsLocalClient(isLocalClient),
 	m_LastConnectionCheck(0),
@@ -89,6 +89,7 @@ CNetClient::CNetClient(CGame* game, bool isLocalClient) :
 
 	AddTransition(NCS_HANDSHAKE, (uint)NMT_SERVER_HANDSHAKE_RESPONSE, NCS_AUTHENTICATE, (void*)&OnHandshakeResponse, context);
 
+	AddTransition(NCS_AUTHENTICATE, (uint)NMT_AUTHENTICATE, NCS_AUTHENTICATE, (void*)&OnAuthenticateRequest, context);
 	AddTransition(NCS_AUTHENTICATE, (uint)NMT_AUTHENTICATE_RESULT, NCS_INITIAL_GAMESETUP, (void*)&OnAuthenticate, context);
 
 	AddTransition(NCS_INITIAL_GAMESETUP, (uint)NMT_GAME_SETUP, NCS_PREGAME, (void*)&OnGameSetup, context);
@@ -159,6 +160,11 @@ void CNetClient::SetUserName(const CStrW& username)
 	m_UserName = username;
 }
 
+void CNetClient::SetHostingPlayerName(const CStr& hostingPlayerName)
+{
+	m_HostingPlayerName = hostingPlayerName;
+}
+
 bool CNetClient::SetupConnection(const CStr& server, const u16 port, ENetHost* enetClient)
 {
 	CNetClientSession* session = new CNetClientSession(*this);
@@ -270,7 +276,7 @@ std::string CNetClient::TestReadGuiMessages()
 	return r;
 }
 
-ScriptInterface& CNetClient::GetScriptInterface()
+const ScriptInterface& CNetClient::GetScriptInterface()
 {
 	return m_Game->GetSimulation2()->GetScriptInterface();
 }
@@ -329,7 +335,7 @@ void CNetClient::HandleDisconnect(u32 reason)
 	SetCurrState(NCS_UNCONNECTED);
 }
 
-void CNetClient::SendGameSetupMessage(JS::MutableHandleValue attrs, ScriptInterface& scriptInterface)
+void CNetClient::SendGameSetupMessage(JS::MutableHandleValue attrs, const ScriptInterface& scriptInterface)
 {
 	CGameSetupMessage gameSetup(scriptInterface);
 	gameSetup.m_Data = attrs;
@@ -477,6 +483,15 @@ void CNetClient::LoadFinished()
 	SendMessage(&loaded);
 }
 
+void CNetClient::SendAuthenticateMessage()
+{
+	CAuthenticateMessage authenticate;
+	authenticate.m_Name = m_UserName;
+	authenticate.m_Password = L""; // TODO
+	authenticate.m_IsLocalClient = m_IsLocalClient;
+	SendMessage(&authenticate);
+}
+
 bool CNetClient::OnConnect(void* context, CFsmEvent* event)
 {
 	ENSURE(event->GetType() == (uint)NMT_CONNECT_COMPLETE);
@@ -514,13 +529,37 @@ bool CNetClient::OnHandshakeResponse(void* context, CFsmEvent* event)
 
 	CNetClient* client = (CNetClient*)context;
 
-	CAuthenticateMessage authenticate;
-	authenticate.m_GUID = client->m_GUID;
-	authenticate.m_Name = client->m_UserName;
-	authenticate.m_Password = L""; // TODO
-	authenticate.m_IsLocalClient = client->m_IsLocalClient;
-	client->SendMessage(&authenticate);
+	CSrvHandshakeResponseMessage* message = (CSrvHandshakeResponseMessage*)event->GetParamRef();
+	client->m_GUID = message->m_GUID;
 
+	if (message->m_Flags & PS_NETWORK_FLAG_REQUIRE_LOBBYAUTH)
+	{
+		if (g_XmppClient && !client->m_HostingPlayerName.empty())
+			g_XmppClient->SendIqLobbyAuth(client->m_HostingPlayerName, client->m_GUID);
+		else
+		{
+			JSContext* cx = client->GetScriptInterface().GetContext();
+			JSAutoRequest rq(cx);
+
+			JS::RootedValue msg(cx);
+			client->GetScriptInterface().Eval("({'type':'netstatus','status':'disconnected'})", &msg);
+			client->GetScriptInterface().SetProperty(msg, "reason", (int)NDR_LOBBY_AUTH_FAILED, false);
+			client->PushGuiMessage(msg);
+			LOGMESSAGE("Net client: Couldn't send lobby auth xmpp message");
+		}
+		return true;
+	}
+
+	client->SendAuthenticateMessage();
+	return true;
+}
+
+bool CNetClient::OnAuthenticateRequest(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_AUTHENTICATE);
+
+	CNetClient* client = (CNetClient*)context;
+	client->SendAuthenticateMessage();
 	return true;
 }
 
