@@ -68,7 +68,7 @@ enum NetDisconnectReason
 	NDR_BANNED,
 	NDR_PLAYERNAME_IN_USE,
 	NDR_SERVER_FULL,
-	NDR_PLAYERGUID_IN_USE
+	NDR_LOBBY_AUTH_FAILED
 };
 
 class CNetHost
