@@ -26,10 +26,13 @@
 #include "ps/CStr.h"
 #include "scriptinterface/ScriptVal.h"
 
-#define PS_PROTOCOL_MAGIC				0x5073013f		// 'P', 's', 0x01, '?'
-#define PS_PROTOCOL_MAGIC_RESPONSE		0x50630121		// 'P', 'c', 0x01, '!'
-#define PS_PROTOCOL_VERSION				0x01010015		// Arbitrary protocol
-#define PS_DEFAULT_PORT					0x5073			// 'P', 's'
+#define PS_PROTOCOL_MAGIC                         0x5073013f	// 'P', 's', 0x01, '?'
+#define PS_PROTOCOL_MAGIC_RESPONSE                0x50630121	// 'P', 'c', 0x01, '!'
+#define PS_PROTOCOL_VERSION                       0x01010015	// Arbitrary protocol
+#define PS_DEFAULT_PORT                           0x5073		// 'P', 's'
+
+// Set when lobby authentication is required. Used in the SrvHandshakeResponseMessage.
+#define PS_NETWORK_FLAG_REQUIRE_LOBBYAUTH         0x1
 
 // Defines the list of message types. The order of the list must not change.
 // The message types having a negative value are used internally and not sent
@@ -113,11 +116,10 @@ END_NMT_CLASS()
 START_NMT_CLASS_(SrvHandshakeResponse, NMT_SERVER_HANDSHAKE_RESPONSE)
 	NMT_FIELD_INT(m_UseProtocolVersion, u32, 4)
 	NMT_FIELD_INT(m_Flags, u32, 4)
-	NMT_FIELD(CStrW, m_Message)
+	NMT_FIELD(CStr, m_GUID)
 END_NMT_CLASS()
 
 START_NMT_CLASS_(Authenticate, NMT_AUTHENTICATE)
-	NMT_FIELD(CStr, m_GUID)
 	NMT_FIELD(CStrW, m_Name)
 	NMT_FIELD(CStrW, m_Password)
 	NMT_FIELD_INT(m_IsLocalClient, u8, 1)
