@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include "network/StunClient.h"
 #include "ps/CLogger.h"
 #include "ps/ConfigDB.h"
+#include "ps/GUID.h"
 #include "ps/Profile.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "scriptinterface/ScriptRuntime.h"
@@ -51,7 +52,6 @@
 #define MAX_CLIENTS 41
 
 #define	DEFAULT_SERVER_NAME			L"Unnamed Server"
-#define DEFAULT_WELCOME_MESSAGE		L"Welcome"
 
 static const int CHANNEL_COUNT = 1;
 
@@ -127,8 +127,9 @@ private:
  * See http://trac.wildfiregames.com/ticket/654
  */
 
-CNetServerWorker::CNetServerWorker(int autostartPlayers) :
+CNetServerWorker::CNetServerWorker(bool useLobbyAuth, int autostartPlayers) :
 	m_AutostartPlayers(autostartPlayers),
+	m_LobbyAuth(useLobbyAuth),
 	m_Shutdown(false),
 	m_ScriptInterface(NULL),
 	m_NextHostID(1), m_Host(NULL), m_HostGUID(), m_Stats(NULL),
@@ -139,7 +140,6 @@ CNetServerWorker::CNetServerWorker(int autostartPlayers) :
 	m_ServerTurnManager = NULL;
 
 	m_ServerName = DEFAULT_SERVER_NAME;
-	m_WelcomeMessage = DEFAULT_WELCOME_MESSAGE;
 }
 
 CNetServerWorker::~CNetServerWorker()
@@ -408,6 +408,7 @@ bool CNetServerWorker::RunStep()
 
 	std::vector<bool> newStartGame;
 	std::vector<std::string> newGameAttributes;
+	std::vector<std::pair<CStr, CStr>> newLobbyAuths;
 	std::vector<u32> newTurnLength;
 
 	{
@@ -418,6 +419,7 @@ bool CNetServerWorker::RunStep()
 
 		newStartGame.swap(m_StartGameQueue);
 		newGameAttributes.swap(m_GameAttributesQueue);
+		newLobbyAuths.swap(m_LobbyAuthQueue);
 		newTurnLength.swap(m_TurnLengthQueue);
 	}
 
@@ -435,6 +437,13 @@ bool CNetServerWorker::RunStep()
 	if (!newStartGame.empty())
 		StartGame();
 
+	while (!newLobbyAuths.empty())
+	{
+		const std::pair<CStr, CStr>& auth = newLobbyAuths.back();
+		ProcessLobbyAuth(auth.first, auth.second);
+		newLobbyAuths.pop_back();
+	}
+
 	// Perform file transfers
 	for (CNetServerSession* session : m_Sessions)
 		session->GetFileTransferer().Poll();
@@ -633,6 +642,9 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 	session->AddTransition(NSS_HANDSHAKE, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED);
 	session->AddTransition(NSS_HANDSHAKE, (uint)NMT_CLIENT_HANDSHAKE, NSS_AUTHENTICATE, (void*)&OnClientHandshake, context);
 
+	session->AddTransition(NSS_LOBBY_AUTHENTICATE, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED);
+	session->AddTransition(NSS_LOBBY_AUTHENTICATE, (uint)NMT_AUTHENTICATE, NSS_PREGAME, (void*)&OnAuthenticate, context);
+
 	session->AddTransition(NSS_AUTHENTICATE, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED);
 	session->AddTransition(NSS_AUTHENTICATE, (uint)NMT_AUTHENTICATE, NSS_PREGAME, (void*)&OnAuthenticate, context);
 
@@ -846,7 +858,7 @@ void CNetServerWorker::SendPlayerAssignments()
 	Broadcast(&message, { NSS_PREGAME, NSS_JOIN_SYNCING, NSS_INGAME });
 }
 
-ScriptInterface& CNetServerWorker::GetScriptInterface()
+const ScriptInterface& CNetServerWorker::GetScriptInterface()
 {
 	return *m_ScriptInterface;
 }
@@ -857,6 +869,24 @@ void CNetServerWorker::SetTurnLength(u32 msecs)
 		m_ServerTurnManager->SetTurnLength(msecs);
 }
 
+void CNetServerWorker::ProcessLobbyAuth(const CStr& name, const CStr& token)
+{
+	LOGMESSAGE("Net Server: Received lobby auth message from %s with %s", name, token);
+	// Find the user with that guid
+	std::vector<CNetServerSession*>::iterator it = std::find_if(m_Sessions.begin(), m_Sessions.end(),
+		[&](CNetServerSession* session)
+		{ return session->GetGUID() == token; });
+
+	if (it == m_Sessions.end())
+		return;
+
+	(*it)->SetUserName(name.FromUTF8());
+	// Send an empty message to request the authentication message from the client
+	// after its identity has been confirmed via the lobby
+	CAuthenticateMessage emptyMessage;
+	(*it)->SendMessage(&emptyMessage);
+}
+
 bool CNetServerWorker::OnClientHandshake(void* context, CFsmEvent* event)
 {
 	ENSURE(event->GetType() == (uint)NMT_CLIENT_HANDSHAKE);
@@ -871,10 +901,35 @@ bool CNetServerWorker::OnClientHandshake(void* context, CFsmEvent* event)
 		return false;
 	}
 
+	CStr guid = ps_generate_guid();
+	int count = 0;
+	// Ensure unique GUID
+	while(std::find_if(
+		server.m_Sessions.begin(), server.m_Sessions.end(),
+		[&guid] (const CNetServerSession* session)
+		{ return session->GetGUID() == guid; }) != server.m_Sessions.end())
+	{
+		if (++count > 100)
+		{
+			session->Disconnect(NDR_UNKNOWN);
+			return true;
+		}
+		guid = ps_generate_guid();
+	}
+
+	session->SetGUID(guid);
+
 	CSrvHandshakeResponseMessage handshakeResponse;
 	handshakeResponse.m_UseProtocolVersion = PS_PROTOCOL_VERSION;
-	handshakeResponse.m_Message = server.m_WelcomeMessage;
+	handshakeResponse.m_GUID = guid;
 	handshakeResponse.m_Flags = 0;
+
+	if (server.m_LobbyAuth)
+	{
+		handshakeResponse.m_Flags |= PS_NETWORK_FLAG_REQUIRE_LOBBYAUTH;
+		session->SetNextState(NSS_LOBBY_AUTHENTICATE);
+	}
+
 	session->SendMessage(&handshakeResponse);
 
 	return true;
@@ -897,31 +952,36 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 
 	CAuthenticateMessage* message = (CAuthenticateMessage*)event->GetParamRef();
 	CStrW username = SanitisePlayerName(message->m_Name);
-	CStr guid = message->m_GUID;
+	CStrW usernameWithoutRating(username.substr(0, username.find(L" (")));
+
+	// Compare the lowercase names as specified by https://xmpp.org/extensions/xep-0029.html#sect-idm139493404168176
+	// "[...] comparisons will be made in case-normalized canonical form."
+	if (server.m_LobbyAuth && usernameWithoutRating.LowerCase() != session->GetUserName().LowerCase())
+	{
+		LOGERROR("Net server: lobby auth: %s tried joining as %s",
+			session->GetUserName().ToUTF8(),
+			usernameWithoutRating.ToUTF8());
+		session->Disconnect(NDR_LOBBY_AUTH_FAILED);
+		return true;
+	}
 
 	// Either deduplicate or prohibit join if name is in use
 	bool duplicatePlayernames = false;
 	CFG_GET_VAL("network.duplicateplayernames", duplicatePlayernames);
 	if (duplicatePlayernames)
 		username = server.DeduplicatePlayerName(username);
-	else if (std::find_if(
+	else
+	{
+		std::vector<CNetServerSession*>::iterator it = std::find_if(
 			server.m_Sessions.begin(), server.m_Sessions.end(),
 			[&username] (const CNetServerSession* session)
-			{ return session->GetUserName() == username; })
-		!= server.m_Sessions.end())
-	{
-		session->Disconnect(NDR_PLAYERNAME_IN_USE);
-		return true;
-	}
+			{ return session->GetUserName() == username; });
 
-	// Disconnect user if the provided GUID is already in use
-	if (std::find_if(
-		server.m_Sessions.begin(), server.m_Sessions.end(),
-		[&guid] (const CNetServerSession* session)
-		{ return session->GetGUID() == guid; }) != server.m_Sessions.end())
-	{
-		session->Disconnect(NDR_PLAYERGUID_IN_USE);
-		return true;
+		if (it != server.m_Sessions.end() && (*it) != session)
+		{
+			session->Disconnect(NDR_PLAYERNAME_IN_USE);
+			return true;
+		}
 	}
 
 	// Disconnect banned usernames
@@ -978,7 +1038,6 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 			}
 			else if (observerLateJoin == "buddies")
 			{
-				CStrW usernameWithoutRating(username.substr(0, username.find(L" (")));
 				CStr buddies;
 				CFG_GET_VAL("lobby.buddies", buddies);
 				std::wstringstream buddiesStream(wstring_from_utf8(buddies));
@@ -1018,7 +1077,6 @@ bool CNetServerWorker::OnAuthenticate(void* context, CFsmEvent* event)
 	u32 newHostID = server.m_NextHostID++;
 
 	session->SetUserName(username);
-	session->SetGUID(message->m_GUID);
 	session->SetHostID(newHostID);
 	session->SetLocalClient(message->m_IsLocalClient);
 
@@ -1063,7 +1121,7 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 		// Ignore messages sent by one player on behalf of another player
 		// unless cheating is enabled
 		bool cheatsEnabled = false;
-		ScriptInterface& scriptInterface = server.GetScriptInterface();
+		const ScriptInterface& scriptInterface = server.GetScriptInterface();
 		JSContext* cx = scriptInterface.GetContext();
 		JSAutoRequest rq(cx);
 		JS::RootedValue settings(cx);
@@ -1091,13 +1149,13 @@ bool CNetServerWorker::OnInGame(void* context, CFsmEvent* event)
 	else if (message->GetType() == (uint)NMT_SYNC_CHECK)
 	{
 		CSyncCheckMessage* syncMessage = static_cast<CSyncCheckMessage*> (message);
-		server.m_ServerTurnManager->NotifyFinishedClientUpdate(session->GetHostID(), session->GetUserName(), syncMessage->m_Turn, syncMessage->m_Hash);
+		server.m_ServerTurnManager->NotifyFinishedClientUpdate(*session, syncMessage->m_Turn, syncMessage->m_Hash);
 	}
 	else if (message->GetType() == (uint)NMT_END_COMMAND_BATCH)
 	{
 		// The turn-length field is ignored
 		CEndCommandBatchMessage* endMessage = static_cast<CEndCommandBatchMessage*> (message);
-		server.m_ServerTurnManager->NotifyFinishedClientCommands(session->GetHostID(), endMessage->m_Turn);
+		server.m_ServerTurnManager->NotifyFinishedClientCommands(*session, endMessage->m_Turn);
 	}
 
 	return true;
@@ -1476,8 +1534,8 @@ void CNetServerWorker::SendHolePunchingMessage(const CStr& ipStr, u16 port)
 
 
 
-CNetServer::CNetServer(int autostartPlayers) :
-	m_Worker(new CNetServerWorker(autostartPlayers))
+CNetServer::CNetServer(bool useLobbyAuth, int autostartPlayers) :
+	m_Worker(new CNetServerWorker(useLobbyAuth, autostartPlayers))
 {
 }
 
@@ -1497,7 +1555,7 @@ void CNetServer::StartGame()
 	m_Worker->m_StartGameQueue.push_back(true);
 }
 
-void CNetServer::UpdateGameAttributes(JS::MutableHandleValue attrs, ScriptInterface& scriptInterface)
+void CNetServer::UpdateGameAttributes(JS::MutableHandleValue attrs, const ScriptInterface& scriptInterface)
 {
 	// Pass the attributes as JSON, since that's the easiest safe
 	// cross-thread way of passing script data
@@ -1507,6 +1565,12 @@ void CNetServer::UpdateGameAttributes(JS::MutableHandleValue attrs, ScriptInterf
 	m_Worker->m_GameAttributesQueue.push_back(attrsJSON);
 }
 
+void CNetServer::OnLobbyAuth(const CStr& name, const CStr& token)
+{
+	CScopeLock lock(m_Worker->m_WorkerMutex);
+	m_Worker->m_LobbyAuthQueue.push_back(std::make_pair(name, token));
+}
+
 void CNetServer::SetTurnLength(u32 msecs)
 {
 	CScopeLock lock(m_Worker->m_WorkerMutex);
