@@ -71,6 +71,9 @@ enum NetServerSessionState
 	// to agree on the protocol version
 	NSS_HANDSHAKE,
 
+	// The client has handshook and we're waiting for its lobby authentication message
+	NSS_LOBBY_AUTHENTICATE,
+
 	// The client has handshook and we're waiting for its authentication message,
 	// to find its name and check its password etc
 	NSS_AUTHENTICATE,
@@ -104,7 +107,7 @@ public:
 	 * @param autostartPlayers if positive then StartGame will be called automatically
 	 * once this many players are connected (intended for the command-line testing mode).
 	 */
-	CNetServer(int autostartPlayers = -1);
+	CNetServer(bool useLobbyAuth = false, int autostartPlayers = -1);
 
 	~CNetServer();
 
@@ -126,7 +129,7 @@ public:
 	 * The changes will be asynchronously propagated to all clients.
 	 * @param attrs game attributes, in the script context of scriptInterface
 	 */
-	void UpdateGameAttributes(JS::MutableHandleValue attrs, ScriptInterface& scriptInterface);
+	void UpdateGameAttributes(JS::MutableHandleValue attrs, const ScriptInterface& scriptInterface);
 
 	/**
 	 * Set the turn length to a fixed value.
@@ -134,6 +137,8 @@ public:
 	 */
 	void SetTurnLength(u32 msecs);
 
+	void OnLobbyAuth(const CStr& name, const CStr& token);
+
 	void SendHolePunchingMessage(const CStr& ip, u16 port);
 
 private:
@@ -178,7 +183,7 @@ private:
 	friend class CNetServer;
 	friend class CNetFileReceiveTask_ServerRejoin;
 
-	CNetServerWorker(int autostartPlayers);
+	CNetServerWorker(bool useLobbyAuth, int autostartPlayers);
 	~CNetServerWorker();
 
 	/**
@@ -221,7 +226,7 @@ private:
 	/**
 	 * Get the script context used for game attributes.
 	 */
-	ScriptInterface& GetScriptInterface();
+	const ScriptInterface& GetScriptInterface();
 
 	/**
 	 * Set the turn length to a fixed value.
@@ -229,6 +234,8 @@ private:
 	 */
 	void SetTurnLength(u32 msecs);
 
+	void ProcessLobbyAuth(const CStr& name, const CStr& token);
+
 	void AddPlayer(const CStr& guid, const CStrW& name);
 	void RemovePlayer(const CStr& guid);
 	void SendPlayerAssignments();
@@ -291,6 +298,7 @@ private:
 	JS::PersistentRootedValue m_GameAttributes;
 
 	int m_AutostartPlayers;
+	bool m_LobbyAuth;
 
 	ENetHost* m_Host;
 	std::vector<CNetServerSession*> m_Sessions;
@@ -300,7 +308,6 @@ private:
 	NetServerState m_State;
 
 	CStrW m_ServerName;
-	CStrW m_WelcomeMessage;
 
 	std::vector<u32> m_BannedIPs;
 	std::vector<CStrW> m_BannedPlayers;
@@ -352,12 +359,14 @@ private:
 	pthread_t m_WorkerThread;
 	CMutex m_WorkerMutex;
 
-	bool m_Shutdown; // protected by m_WorkerMutex
+	// protected by m_WorkerMutex
+	bool m_Shutdown;
 
-	// Queues for messages sent by the game thread:
-	std::vector<bool> m_StartGameQueue; // protected by m_WorkerMutex
-	std::vector<std::string> m_GameAttributesQueue; // protected by m_WorkerMutex
-	std::vector<u32> m_TurnLengthQueue; // protected by m_WorkerMutex
+	// Queues for messages sent by the game thread (protected by m_WorkerMutex):
+	std::vector<bool> m_StartGameQueue;
+	std::vector<std::string> m_GameAttributesQueue;
+	std::vector<std::pair<CStr, CStr>> m_LobbyAuthQueue;
+	std::vector<u32> m_TurnLengthQueue;
 };
 
 /// Global network server for the standard game
