@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -27,13 +27,6 @@
 #include "lib/res/h_mgr.h"
 #include "lib/res/graphics/cursor.h"
 #include "lib/sysdep/cursor.h"
-#include "lib/sysdep/cpu.h"
-#include "lib/sysdep/gfx.h"
-#include "lib/sysdep/os_cpu.h"
-#include "lib/tex/tex.h"
-#if OS_WIN
-#include "lib/sysdep/os/win/wversion.h"
-#endif
 
 #include "graphics/CinemaManager.h"
 #include "graphics/FontMetrics.h"
@@ -68,6 +61,7 @@
 #include "ps/Joystick.h"
 #include "ps/Loader.h"
 #include "ps/Mod.h"
+#include "ps/ModIo.h"
 #include "ps/Profile.h"
 #include "ps/ProfileViewer.h"
 #include "ps/Profiler2.h"
@@ -100,7 +94,7 @@
 #define MUST_INIT_X11 0
 #endif
 
-extern void restart_engine();
+extern void RestartEngine();
 
 #include <iostream>
 
@@ -337,56 +331,6 @@ void Render()
 	ogl_WarnIfError();
 }
 
-
-static size_t OperatingSystemFootprint()
-{
-#if OS_WIN
-	switch(wversion_Number())
-	{
-	case WVERSION_2K:
-	case WVERSION_XP:
-		return 150;
-	case WVERSION_XP64:
-		return 200;
-	default:	// newer Windows version: assume the worst, and don't warn
-	case WVERSION_VISTA:
-		return 300;
-	case WVERSION_7:
-		return 250;
-	}
-#else
-	return 200;
-#endif
-}
-
-static size_t ChooseCacheSize()
-{
-	// (all sizes in MiB and signed to allow temporarily negative computations)
-
-	const ssize_t total = (ssize_t)os_cpu_MemorySize();
-	// (NB: os_cpu_MemoryAvailable is useless on Linux because free memory
-	// is marked as "in use" by OS caches.)
-	const ssize_t os = (ssize_t)OperatingSystemFootprint();
-	const ssize_t game = 300;	// estimated working set
-
-	ssize_t cache = 400;	// upper bound: total size of our data
-
-	// the cache reserves contiguous address space, which is a precious
-	// resource on 32-bit systems, so don't use too much:
-	if(ARCH_IA32 || sizeof(void*) == 4)
-		cache = std::min(cache, (ssize_t)200);
-
-	// try to leave over enough memory for the OS and game
-	cache = std::min(cache, total-os-game);
-
-	// always provide at least this much to ensure correct operation
-	cache = std::max(cache, (ssize_t)64);
-
-	debug_printf("Cache: %d (total: %d) MiB\n", (int)cache, (int)total);
-	return size_t(cache)*MiB;
-}
-
-
 ErrorReactionInternal psDisplayError(const wchar_t* UNUSED(text), size_t UNUSED(flags))
 {
 	// If we're fullscreen, then sometimes (at least on some particular drivers on Linux)
@@ -408,7 +352,7 @@ ErrorReactionInternal psDisplayError(const wchar_t* UNUSED(text), size_t UNUSED(
 	return ERI_NOT_IMPLEMENTED;
 }
 
-std::vector<CStr>& GetMods(const CmdLineArgs& args, int flags)
+const std::vector<CStr>& GetMods(const CmdLineArgs& args, int flags)
 {
 	const bool init_mods = (flags & INIT_MODS) == INIT_MODS;
 	const bool add_user = !InDevelopmentCopy() && !args.Has("noUserMod");
@@ -488,8 +432,7 @@ static void InitVfs(const CmdLineArgs& args, int flags)
 		hooks.display_error = psDisplayError;
 	app_hooks_update(&hooks);
 
-	const size_t cacheSize = ChooseCacheSize();
-	g_VFS = CreateVfs(cacheSize);
+	g_VFS = CreateVfs();
 
 	const OsPath readonlyConfig = paths.RData()/"config"/"";
 	g_VFS->Mount(L"config/", readonlyConfig);
@@ -553,6 +496,32 @@ static void InitPs(bool setup_gui, const CStrW& gui_page, ScriptInterface* srcSc
 	g_GUI->SwitchPage(gui_page, srcScriptInterface, initData);
 }
 
+void InitPsAutostart(bool networked, JS::HandleValue attrs)
+{
+	// The GUI has not been initialized yet, so use the simulation scriptinterface for this variable
+	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+
+	JS::RootedValue playerAssignments(cx);
+	scriptInterface.Eval("({})", &playerAssignments);
+
+	if (!networked)
+	{
+		JS::RootedValue localPlayer(cx);
+		scriptInterface.Eval("({})", &localPlayer);
+		scriptInterface.SetProperty(localPlayer, "player", g_Game->GetPlayerID());
+		scriptInterface.SetProperty(playerAssignments, "local", localPlayer);
+	}
+
+	JS::RootedValue sessionInitData(cx);
+	scriptInterface.Eval("({})", &sessionInitData);
+	scriptInterface.SetProperty(sessionInitData, "attribs", attrs);
+	scriptInterface.SetProperty(sessionInitData, "playerAssignments", playerAssignments);
+
+	InitPs(true, L"page_loading.xml", &scriptInterface, sessionInitData);
+}
+
 
 static void InitInput()
 {
@@ -580,8 +549,6 @@ static void InitInput()
 
 	in_add_handler(touch_input_handler);
 
-	in_add_handler(cinema_manager_handler);
-
 	// must be registered after (called before) the GUI which relies on these globals
 	in_add_handler(GlobalsInputHandler);
 }
@@ -616,8 +583,8 @@ static void InitRenderer()
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHADOWS, g_Shadows);
 	g_ConfigDB.SetValueBool(CFG_SYSTEM, "shadows", g_Shadows);
 
-	g_Renderer.SetOptionBool(CRenderer::OPT_WATERUGLY, g_WaterUgly);
-	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterugly", g_WaterUgly);
+	g_Renderer.SetOptionBool(CRenderer::OPT_WATEREFFECTS, g_WaterEffects);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "watereffects", g_WaterEffects);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERFANCYEFFECTS, g_WaterFancyEffects);
 	g_ConfigDB.SetValueBool(CFG_SYSTEM, "waterfancyeffects", g_WaterFancyEffects);
 	g_Renderer.SetOptionBool(CRenderer::OPT_WATERREALDEPTH, g_WaterRealDepth);
@@ -634,6 +601,8 @@ static void InitRenderer()
 	g_ConfigDB.SetValueBool(CFG_SYSTEM, "shadowpcf", g_ShadowPCF);
 	g_Renderer.SetOptionBool(CRenderer::OPT_PARTICLES, g_Particles);
 	g_ConfigDB.SetValueBool(CFG_SYSTEM, "particles", g_Particles);
+	g_Renderer.SetOptionBool(CRenderer::OPT_FOG, g_Fog);
+	g_ConfigDB.SetValueBool(CFG_SYSTEM, "fog", g_Fog);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SILHOUETTES, g_Silhouettes);
 	g_ConfigDB.SetValueBool(CFG_SYSTEM, "silhouettes", g_Silhouettes);
 	g_Renderer.SetOptionBool(CRenderer::OPT_SHOWSKY, g_ShowSky);
@@ -736,6 +705,8 @@ void Shutdown(int flags)
 
 	SAFE_DELETE(g_XmppClient);
 
+	SAFE_DELETE(g_ModIo);
+
 	ShutdownPs();
 
 	TIMER_BEGIN(L"shutdown TexMan");
@@ -767,6 +738,9 @@ void Shutdown(int flags)
 	g_UserReporter.Deinitialize();
 	TIMER_END(L"shutdown UserReporter");
 
+	// Cleanup curl now that g_ModIo and g_UserReporter have been shutdown.
+	curl_global_cleanup();
+
 	delete &g_L10n;
 
 from_config:
@@ -984,7 +958,7 @@ bool Init(const CmdLineArgs& args, int flags)
 			std::swap(g_modsLoaded, mods);
 
 			// Abort init and restart
-			restart_engine();
+			RestartEngine();
 			return false;
 		}
 	}
@@ -998,6 +972,10 @@ bool Init(const CmdLineArgs& args, int flags)
 	if (profilerHTTPEnable)
 		g_Profiler2.EnableHTTP();
 
+	// Initialise everything except Win32 sockets (because our networking
+	// system already inits those)
+	curl_global_init(CURL_GLOBAL_ALL & ~CURL_GLOBAL_WIN32);
+
 	if (!g_Quickstart)
 		g_UserReporter.Initialize(); // after config
 
@@ -1005,7 +983,7 @@ bool Init(const CmdLineArgs& args, int flags)
 	return true;
 }
 
-void InitGraphics(const CmdLineArgs& args, int flags)
+void InitGraphics(const CmdLineArgs& args, int flags, const std::vector<CStr>& installedMods)
 {
 	const bool setup_vmode = (flags & INIT_HAVE_VMODE) == 0;
 
@@ -1109,8 +1087,10 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 			{
 				scriptInterface->Eval("({})", &data);
 				scriptInterface->SetProperty(data, "isStartup", true);
+				if (!installedMods.empty())
+					scriptInterface->SetProperty(data, "installedMods", installedMods);
 			}
-			InitPs(setup_gui, L"page_pregame.xml", g_GUI->GetScriptInterface().get(), data);
+			InitPs(setup_gui, installedMods.empty() ? L"page_pregame.xml" : L"page_modmod.xml", g_GUI->GetScriptInterface().get(), data);
 		}
 	}
 	catch (PSERROR_Game_World_MapLoadFailed& e)
@@ -1207,13 +1187,22 @@ CStr8 LoadSettingsOfScenarioMap(const VfsPath &mapPath)
  *                                 (0: sandbox, 5: very hard)
  * -autostart-aiseed=AISEED        sets the seed used for the AI random
  *                                 generator (default 0, use -1 for random)
+ * -autostart-player=NUMBER        sets the playerID in non-networked games (default 1, use -1 for observer)
  * -autostart-civ=PLAYER:CIV       sets PLAYER's civilisation to CIV
  *                                 (skirmish and random maps only)
  * -autostart-team=PLAYER:TEAM     sets the team for PLAYER (e.g. 2:2).
+ * -autostart-ceasefire=NUM        sets a ceasefire duration NUM
+ *                                 (default 0 minutes)
  * -autostart-nonvisual            disable any graphics and sounds
  * -autostart-victory=SCRIPTNAME   sets the victory conditions with SCRIPTNAME
  *                                 located in simulation/data/settings/victory_conditions/
- * -autostart-victoryduration=NUM  sets the victory duration NUM for specific victory conditions
+ *                                 (default conquest)
+ * -autostart-wonderduration=NUM   sets the victory duration NUM for wonder victory condition
+ *                                 (default 10 minutes)
+ * -autostart-relicduration=NUM    sets the victory duration NUM for relic victory condition
+ *                                 (default 10 minutes)
+ * -autostart-reliccount=NUM       sets the number of relics for relic victory condition
+ *                                 (default 2 relics)
  *
  * Multiplayer:
  * -autostart-playername=NAME      sets local player NAME (default 'anonymous')
@@ -1230,10 +1219,16 @@ CStr8 LoadSettingsOfScenarioMap(const VfsPath &mapPath)
  * Examples:
  * 1) "Bob" will host a 2 player game on the Arcadia map:
  * -autostart="scenarios/Arcadia" -autostart-host -autostart-host-players=2 -autostart-playername="Bob"
+ *  "Alice" joins the match as player 2:
+ * -autostart="scenarios/Arcadia" -autostart-client=127.0.0.1 -autostart-playername="Alice"
+ * The players use the developer overlay to control players.
  *
  * 2) Load Alpine Lakes random map with random seed, 2 players (Athens and Britons), and player 2 is PetraBot:
  * -autostart="random/alpine_lakes" -autostart-seed=-1 -autostart-players=2 -autostart-civ=1:athen -autostart-civ=2:brit -autostart-ai=2:petra
-*/
+ *
+ * 3) Observe the PetraBot on a triggerscript map:
+ * -autostart="random/jebel_barkal" -autostart-seed=-1 -autostart-players=2 -autostart-civ=1:athen -autostart-civ=2:brit -autostart-ai=1:petra -autostart-ai=2:petra -autostart-player=-1
+ */
 bool Autostart(const CmdLineArgs& args)
 {
 	CStr autoStartName = args.Get("autostart");
@@ -1402,6 +1397,11 @@ bool Autostart(const CmdLineArgs& args)
 		}
 	}
 
+	int ceasefire = 0;
+	if (args.Has("autostart-ceasefire"))
+		ceasefire = args.Get("autostart-ceasefire").ToInt();
+	scriptInterface.SetProperty(settings, "Ceasefire", ceasefire);
+
 	if (args.Has("autostart-ai"))
 	{
 		std::vector<CStr> aiArgs = args.GetMultiple("autostart-ai");
@@ -1425,6 +1425,7 @@ bool Autostart(const CmdLineArgs& args)
 			CStr name = aiArgs[i].AfterFirst(":");
 			scriptInterface.SetProperty(player, "AI", std::string(name));
 			scriptInterface.SetProperty(player, "AIDiff", 3);
+			scriptInterface.SetProperty(player, "AIBehavior", std::string("balanced"));
 			scriptInterface.SetPropertyInt(playerData, playerID-offset, player);
 		}
 	}
@@ -1492,10 +1493,6 @@ bool Autostart(const CmdLineArgs& args)
 	// Add map settings to game attributes
 	scriptInterface.SetProperty(attrs, "settings", settings);
 
-	JS::RootedValue mpInitData(cx);
-	scriptInterface.Eval("({isNetworked:true, playerAssignments:{}})", &mpInitData);
-	scriptInterface.SetProperty(mpInitData, "attribs", attrs);
-
 	// Get optional playername
 	CStrW userName = L"anonymous";
 	if (args.Has("autostart-playername"))
@@ -1517,40 +1514,64 @@ bool Autostart(const CmdLineArgs& args)
 		triggerScriptsVector.push_back(nonVisualScript.FromUTF8());
 	}
 
+	std::vector<CStr> victoryConditions(1, "conquest");
 	if (args.Has("autostart-victory"))
+		victoryConditions = args.GetMultiple("autostart-victory");
+
+	if (victoryConditions.size() == 1 && victoryConditions[0] == "endless")
+		victoryConditions.clear();
+
+	scriptInterface.SetProperty(settings, "VictoryConditions", victoryConditions);
+
+	for (const CStr& victory : victoryConditions)
 	{
-		CStrW scriptName = args.Get("autostart-victory").FromUTF8();
-		CStrW scriptPath = L"simulation/data/settings/victory_conditions/" + scriptName + L".json";
 		JS::RootedValue scriptData(cx);
 		JS::RootedValue data(cx);
 		JS::RootedValue victoryScripts(cx);
 
+		CStrW scriptPath = L"simulation/data/settings/victory_conditions/" + victory.FromUTF8() + L".json";
 		scriptInterface.ReadJSONFile(scriptPath, &scriptData);
-
 		if (!scriptData.isUndefined() && scriptInterface.GetProperty(scriptData, "Data", &data) && !data.isUndefined()
-				&& scriptInterface.GetProperty(data, "Scripts", &victoryScripts) && !victoryScripts.isUndefined())
+			&& scriptInterface.GetProperty(data, "Scripts", &victoryScripts) && !victoryScripts.isUndefined())
 		{
 			std::vector<CStrW> victoryScriptsVector;
 			FromJSVal_vector(cx, victoryScripts, victoryScriptsVector);
 			triggerScriptsVector.insert(triggerScriptsVector.end(), victoryScriptsVector.begin(), victoryScriptsVector.end());
 		}
+		else
+		{
+			LOGERROR("Autostart: Error reading victory script '%s'", utf8_from_wstring(scriptPath));
+			throw PSERROR_Game_World_MapLoadFailed("Error reading victory script.\nCheck application log for details.");
+		}
 	}
 
 	ToJSVal_vector(cx, &triggerScripts, triggerScriptsVector);
 	scriptInterface.SetProperty(settings, "TriggerScripts", triggerScripts);
 
-	if (args.Has("autostart-victoryduration"))
-		scriptInterface.SetProperty(settings, "VictoryDuration", args.Get("autostart-victoryduration").ToInt());
+	int wonderDuration = 10;
+	if (args.Has("autostart-wonderduration"))
+		wonderDuration = args.Get("autostart-wonderduration").ToInt();
+	scriptInterface.SetProperty(settings, "WonderDuration", wonderDuration);
+
+	int relicDuration = 10;
+	if (args.Has("autostart-relicduration"))
+		relicDuration = args.Get("autostart-relicduration").ToInt();
+	scriptInterface.SetProperty(settings, "RelicDuration", relicDuration);
+
+	int relicCount = 2;
+	if (args.Has("autostart-reliccount"))
+		relicCount = args.Get("autostart-reliccount").ToInt();
+	scriptInterface.SetProperty(settings, "RelicCount", relicCount);
 
 	if (args.Has("autostart-host"))
 	{
-		InitPs(true, L"page_loading.xml", &scriptInterface, mpInitData);
+		InitPsAutostart(true, attrs);
 
 		size_t maxPlayers = 2;
 		if (args.Has("autostart-host-players"))
 			maxPlayers = args.Get("autostart-host-players").ToUInt();
 
-		g_NetServer = new CNetServer(maxPlayers);
+		g_NetServer = new CNetServer(false, maxPlayers);
 
 		g_NetServer->UpdateGameAttributes(&attrs, scriptInterface);
 
@@ -1563,7 +1584,7 @@ bool Autostart(const CmdLineArgs& args)
 	}
 	else if (args.Has("autostart-client"))
 	{
-		InitPs(true, L"page_loading.xml", &scriptInterface, mpInitData);
+		InitPsAutostart(true, attrs);
 
 		g_NetClient = new CNetClient(g_Game, false);
 		g_NetClient->SetUserName(userName);
@@ -1577,18 +1598,18 @@ bool Autostart(const CmdLineArgs& args)
 	}
 	else
 	{
-		g_Game->SetPlayerID(1);
+		g_Game->SetPlayerID(args.Has("autostart-player") ? args.Get("autostart-player").ToInt() : 1);
+
 		g_Game->StartGame(&attrs, "");
 
-		LDR_NonprogressiveLoad();
-
-		PSRETURN ret = g_Game->ReallyStartGame();
-		ENSURE(ret == PSRETURN_OK);
-
 		if (nonVisual)
-			return true;
-
-		InitPs(true, L"page_session.xml", NULL, JS::UndefinedHandleValue);
+		{
+			// TODO: Non progressive load can fail - need a decent way to handle this
+			LDR_NonprogressiveLoad();
+			ENSURE(g_Game->ReallyStartGame() == PSRETURN_OK);
+		}
+		else
+			InitPsAutostart(false, attrs);
 	}
 
 	return true;
@@ -1603,14 +1624,13 @@ bool AutostartVisualReplay(const std::string& replayFile)
 	g_Game->SetPlayerID(-1);
 	g_Game->StartVisualReplay(replayFile);
 
-	// TODO: Non progressive load can fail - need a decent way to handle this
-	LDR_NonprogressiveLoad();
-
-	ENSURE(g_Game->ReallyStartGame() == PSRETURN_OK);
-
 	ScriptInterface& scriptInterface = g_Game->GetSimulation2()->GetScriptInterface();
+	JSContext* cx = scriptInterface.GetContext();
+	JSAutoRequest rq(cx);
+	JS::RootedValue attrs(cx, g_Game->GetSimulation2()->GetInitAttributes());
+
+	InitPsAutostart(false, attrs);
 
-	InitPs(true, L"page_session.xml", &scriptInterface, JS::UndefinedHandleValue);
 	return true;
 }
 
