@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2017 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -58,7 +58,7 @@ CWorld::CWorld(CGame *pGame):
 	m_pGame(pGame),
 	m_Terrain(new CTerrain()),
 	m_UnitManager(new CUnitManager()),
-	m_TerritoryManager(NULL)
+	m_MapReader(new CMapReader)
 {
 }
 
@@ -71,13 +71,11 @@ void CWorld::RegisterInit(const CStrW& mapFile, JSRuntime* rt, JS::HandleValue s
 	if (mapFile.length())
 	{
 		VfsPath mapfilename = VfsPath(mapFile).ChangeExtension(L".pmp");
-		CMapReader* reader = 0;
 
 		try
 		{
-			reader = new CMapReader;
 			CTriggerManager* pTriggerManager = NULL;
-			reader->LoadMap(mapfilename, rt, settings, m_Terrain,
+			m_MapReader->LoadMap(mapfilename, rt, settings, m_Terrain,
 				CRenderer::IsInitialised() ? g_Renderer.GetWaterManager() : NULL,
 				CRenderer::IsInitialised() ? g_Renderer.GetSkyManager() : NULL,
 				&g_LightEnv, m_pGame->GetView(),
@@ -85,10 +83,11 @@ void CWorld::RegisterInit(const CStrW& mapFile, JSRuntime* rt, JS::HandleValue s
 				pTriggerManager, CRenderer::IsInitialised() ? &g_Renderer.GetPostprocManager() : NULL,
 				m_pGame->GetSimulation2(), &m_pGame->GetSimulation2()->GetSimContext(), playerID, false);
 				// fails immediately, or registers for delay loading
+			RegMemFun(this, &CWorld::DeleteMapReader, L"CWorld::DeleteMapReader", 5);
 		}
 		catch (PSERROR_File& err)
 		{
-			delete reader;
+			SAFE_DELETE(m_MapReader);
 			LOGERROR("Failed to load map %s: %s", mapfilename.string8(), err.what());
 			throw PSERROR_Game_World_MapLoadFailed("Failed to load map.\nCheck application log for details.");
 		}
@@ -98,11 +97,8 @@ void CWorld::RegisterInit(const CStrW& mapFile, JSRuntime* rt, JS::HandleValue s
 void CWorld::RegisterInitRMS(const CStrW& scriptFile, JSRuntime* rt, JS::HandleValue settings, int playerID)
 {
 	// If scriptFile is empty, a blank map will be generated using settings (no RMS run)
-	CMapReader* reader = 0;
-
-	reader = new CMapReader;
 	CTriggerManager* pTriggerManager = NULL;
-	reader->LoadRandomMap(scriptFile, rt, settings, m_Terrain,
+	m_MapReader->LoadRandomMap(scriptFile, rt, settings, m_Terrain,
 		CRenderer::IsInitialised() ? g_Renderer.GetWaterManager() : NULL,
 		CRenderer::IsInitialised() ? g_Renderer.GetSkyManager() : NULL,
 		&g_LightEnv, m_pGame->GetView(),
@@ -110,8 +106,14 @@ void CWorld::RegisterInitRMS(const CStrW& scriptFile, JSRuntime* rt, JS::HandleV
 		pTriggerManager, CRenderer::IsInitialised() ? &g_Renderer.GetPostprocManager() : NULL,
 		m_pGame->GetSimulation2(), playerID);
 		// registers for delay loading
+	RegMemFun(this, &CWorld::DeleteMapReader, L"CWorld::DeleteMapReader", 5);
 }
 
+int CWorld::DeleteMapReader()
+{
+	SAFE_DELETE(m_MapReader);
+	return 0;
+}
 
 /**
  * Destructor.
@@ -121,4 +123,5 @@ CWorld::~CWorld()
 {
 	delete m_Terrain;
 	delete m_UnitManager;
+	delete m_MapReader;
 }
