@@ -18,7 +18,6 @@
 #ifndef INCLUDED_JSI_VISUALREPLAY
 #define INCLUDED_JSI_VISUALREPLAY
 
-#include "ps/VisualReplay.h"
 #include "scriptinterface/ScriptInterface.h"
 
 namespace JSI_VisualReplay
@@ -30,8 +29,8 @@ namespace JSI_VisualReplay
 	bool HasReplayMetadata(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& directoryName);
 	JS::Value GetReplayMetadata(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& directoryName);
 	void AddReplayToCache(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& directoryName);
-	void RegisterScriptFunctions(ScriptInterface& scriptInterface);
+	void RegisterScriptFunctions(const ScriptInterface& scriptInterface);
 	CStrW GetReplayDirectoryName(ScriptInterface::CxPrivate* pCxPrivate, const CStrW& directoryName);
 }
 
-#endif
+#endif // INCLUDED_JSI_VISUALREPLAY
