@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include "ICmpUnitMotion.h"
 #include "ICmpValueModificationManager.h"
 #include "ICmpVisibility.h"
+#include "ICmpSound.h"
 
 #include "simulation2/serialization/SerializeTemplates.h"
 
@@ -206,7 +207,7 @@ public:
 
 		InitModel(paramNode);
 
-		SelectAnimation("idle", false, fixed::FromInt(1), L"");
+		SelectAnimation("idle");
 	}
 
 	virtual void Deinit()
@@ -387,10 +388,10 @@ public:
 		return m_Unit->GetObject().m_ProjectileModelName;
 	}
 
-	virtual CVector3D GetProjectileLaunchPoint() const
+	virtual CFixedVector3D GetProjectileLaunchPoint() const
 	{
 		if (!m_Unit)
-			return CVector3D();
+			return CFixedVector3D();
 
 		if (m_Unit->GetModel().ToCModel())
 		{
@@ -407,18 +408,28 @@ public:
 
 			CModelAbstract* ammo = m_Unit->GetModel().ToCModel()->FindFirstAmmoProp();
 			if (ammo)
-				return ammo->GetTransform().GetTranslation();
+			{
+				CVector3D vector = ammo->GetTransform().GetTranslation();
+				return CFixedVector3D(fixed::FromFloat(vector.X), fixed::FromFloat(vector.Y), fixed::FromFloat(vector.Z));
+			}
 		}
 
-		return CVector3D();
+		return CFixedVector3D();
 	}
 
 	virtual void SetVariant(const CStr& key, const CStr& selection)
 	{
+		if (m_VariantSelections[key] == selection)
+			return;
+
 		m_VariantSelections[key] = selection;
 
 		if (m_Unit)
+		{
 			m_Unit->SetEntitySelection(key, selection);
+			if (m_Unit->GetAnimation())
+				m_Unit->GetAnimation()->ReloadAnimation();
+		}
 	}
 
 	virtual std::string GetAnimationName() const
@@ -426,21 +437,27 @@ public:
 		return m_AnimName;
 	}
 
-	virtual void SelectAnimation(const std::string& name, bool once, fixed speed, const std::wstring& soundgroup)
+	virtual void SelectAnimation(const std::string& name, bool once = false, fixed speed = fixed::FromInt(1))
 	{
 		m_AnimRunThreshold = fixed::Zero();
 		m_AnimName = name;
 		m_AnimOnce = once;
 		m_AnimSpeed = speed;
-		m_SoundGroup = soundgroup;
+		m_SoundGroup = L"";
 		m_AnimDesync = fixed::FromInt(1)/20; // TODO: make this an argument
 		m_AnimSyncRepeatTime = fixed::Zero();
 		m_AnimSyncOffsetTime = fixed::Zero();
 
 		SetVariant("animation", m_AnimName);
 
-		if (m_Unit && m_Unit->GetAnimation())
-			m_Unit->GetAnimation()->SetAnimationState(m_AnimName, m_AnimOnce, m_AnimSpeed.ToFloat(), m_AnimDesync.ToFloat(), m_SoundGroup.c_str());
+		if (!m_Unit || !m_Unit->GetAnimation() || !m_Unit->GetID())
+			return;
+
+		CmpPtr<ICmpSound> cmpSound(GetSimContext(), m_Unit->GetID());
+		if (cmpSound)
+			m_SoundGroup = cmpSound->GetSoundGroup(wstring_from_utf8(m_AnimName));
+
+		m_Unit->GetAnimation()->SetAnimationState(m_AnimName, m_AnimOnce, m_AnimSpeed.ToFloat(), m_AnimDesync.ToFloat(), m_SoundGroup.c_str());	
 	}
 
 	virtual void ReplaceMoveAnimation(const std::string& name, const std::string& replace)
@@ -457,7 +474,6 @@ public:
 
 	virtual void SelectMovementAnimation(fixed runThreshold)
 	{
-		SelectAnimation("walk", false, fixed::FromFloat(1.f), L"");
 		m_AnimRunThreshold = runThreshold;
 	}
 
@@ -785,6 +801,6 @@ void CCmpVisualActor::Update(fixed UNUSED(turnLength))
 
 	// Selecting the animation is going to reset the anim run threshold, so save it
 	fixed runThreshold = m_AnimRunThreshold;
-	SelectAnimation(name, false, speed, L"");
+	SelectAnimation(name, false, speed);
 	m_AnimRunThreshold = runThreshold;
 }
