@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -60,6 +60,11 @@ public:
 
 	virtual bool IsControlPersistent() const = 0;
 
+	/**
+	 * Test whether the front of the obstruction square is in the water and the back is on the shore.
+	 */
+	virtual bool CheckShorePlacement() const = 0;
+
 	/**
 	 * Test whether this entity is colliding with any obstruction that are set to
 	 * block the creation of foundations.
@@ -84,10 +89,22 @@ public:
 	virtual bool CheckDuplicateFoundation() const = 0;
 
 	/**
-	 * Returns a list of units that are colliding with this entity,
-	 * @return vector of blocking units
+	 * Returns a list of entities that have an obstruction matching the given flag and intersect the current obstruction.
+	 * @return vector of blocking entities
 	 */
-	virtual std::vector<entity_id_t> GetUnitCollisions() const = 0;
+	virtual std::vector<entity_id_t> GetEntitiesByFlags(ICmpObstructionManager::flags_t flags) const = 0;
+
+	/**
+	 * Returns a list of entities that are blocking construction of a foundation.
+	 * @return vector of blocking entities
+	 */
+	virtual std::vector<entity_id_t> GetEntitiesBlockingConstruction() const = 0;
+
+	/**
+	 * Returns a list of entities that shall be deleted when a construction on this obstruction starts,
+	 * for example sheep carcasses.
+	 */
+	virtual std::vector<entity_id_t> GetEntitiesDeletedUponConstruction() const = 0;
 
 	/**
 	 * Detects collisions between foundation-blocking entities and
