@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2015 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -40,13 +40,13 @@ public:
 	virtual bool InitOggFile(const VfsPath& itemPath);
 	virtual bool IsFileFinished();
 	virtual bool IsOneShot();
+	virtual bool IsStereo();
 
 	virtual int FetchDataIntoBuffer(int count, ALuint* buffers);
 	virtual void ResetFile();
 
 protected:
 	OggStreamPtr  ogg;
-//	int m_current_section;
 	bool m_FileFinished;
 	bool m_OneShot;
 	ALuint m_Buffer[100];
