@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Wildfire Games.
+/* Copyright (C) 2018 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -335,6 +335,9 @@ enum
 	ID_DumpState,
 	ID_DumpBinaryState,
 
+	ID_Manual,
+	ID_ReportBug,
+
 	ID_Toolbar // must be last in the list
 };
 
@@ -366,7 +369,10 @@ BEGIN_EVENT_TABLE(ScenarioEditor, wxFrame)
 	EVT_MENU(ID_RenderPathFixed, ScenarioEditor::OnRenderPath)
 	EVT_MENU(ID_RenderPathShader, ScenarioEditor::OnRenderPath)
 
-    EVT_MENU_OPEN(ScenarioEditor::OnMenuOpen)
+	EVT_MENU(ID_Manual, ScenarioEditor::OnHelp)
+	EVT_MENU(ID_ReportBug, ScenarioEditor::OnHelp)
+
+	EVT_MENU_OPEN(ScenarioEditor::OnMenuOpen)
 
 	EVT_IDLE(ScenarioEditor::OnIdle)
 END_EVENT_TABLE()
@@ -487,6 +493,38 @@ ScenarioEditor::ScenarioEditor(wxWindow* parent)
 		menuRP->Append(ID_RenderPathShader, _("&Shader (default)"));
 	}
 
+	wxMenu *menuHelp = new wxMenu;
+	menuBar->Append(menuHelp, _("&Help"));
+	{
+		wxFileName helpPath (_T("tools/atlas/"));
+		helpPath.MakeAbsolute(Datafile::GetDataDirectory());
+		helpPath.SetFullName("help.json");
+		if (wxFileExists(helpPath.GetFullPath()))
+		{
+			wxFFile helpFile(helpPath.GetFullPath());
+			wxString helpData;
+			helpFile.ReadAll(&helpData);
+			AtObj data = AtlasObject::LoadFromJSON(std::string(helpData));
+#define ADD_HELP_ITEM(id) \
+				do { \
+					if (!data[#id].hasContent()) \
+						break; \
+					if (!data[#id]["title"].hasContent() || !data[#id]["url"].hasContent()) \
+						break; \
+					menuHelp->Append(ID_##id, _(wxString(data[#id]["title"])), _(wxString(data[#id]["tooltip"]))); \
+					m_HelpData.insert(std::make_pair( \
+						ID_##id, \
+						HelpItem(wxString(data[#id]["title"]), wxString(data[#id]["tooltip"]), wxString(data[#id]["url"])) \
+					)); \
+				} while (0)
+			ADD_HELP_ITEM(Manual);
+			ADD_HELP_ITEM(ReportBug);
+#undef ADD_HELP_ITEM
+		}
+		else
+			wxLogError(_("'%ls' does not exist"), helpPath.GetFullPath().c_str());
+	}
+
 	m_FileHistory.LoadFromSubDir(*wxConfigBase::Get());
 
 
@@ -723,9 +761,9 @@ void ScenarioEditor::OnOpen(wxCommandEvent& WXUNUSED(event))
 	MapDialog dlg (NULL, MAPDIALOG_OPEN, m_Icon);
 	if (dlg.ShowModal() == wxID_OK)
 	{
-		wxString filename = dlg.GetFilename();
-		if (!OpenFile(filename, filename))
-			wxLogError(_("Map '%ls' does not exist"), filename.c_str());
+		wxString filePath = dlg.GetSelectedFilePath();
+		if (!OpenFile(filePath, filePath))
+			wxLogError(_("Map '%ls' does not exist"), filePath.c_str());
 	}
 
 	// TODO: Make this a non-undoable command
@@ -738,7 +776,7 @@ void ScenarioEditor::OnImportHeightmap(wxCommandEvent& WXUNUSED(event))
 
 	wxFileDialog dlg (NULL, wxFileSelectorPromptStr,
 		_T(""), _T(""),
-		_T("Valid Image files|*.png;*.jpg;*.bmp|All files (*.*)|*.*"),
+		_T("Valid image files (*.png, *.bmp)|*.png;*.bmp|All files (*.*)|*.*"),
 		wxFD_OPEN);
 	// Set default filter
 	dlg.SetFilterIndex(0);
@@ -808,16 +846,16 @@ void ScenarioEditor::OnSaveAs(wxCommandEvent& WXUNUSED(event))
 	MapDialog dlg(NULL, MAPDIALOG_SAVE, m_Icon);
 	if (dlg.ShowModal() == wxID_OK)
 	{
-		wxString filename(dlg.GetFilename());
-		wxBusyInfo busy(_("Saving ") + filename);
+		wxString filePath(dlg.GetSelectedFilePath());
+		wxBusyInfo busy(_("Saving ") + filePath);
 		wxBusyCursor busyc;
 
 		m_ToolManager.SetCurrentTool(_T(""));
 
-		std::wstring map(filename.wc_str());
+		std::wstring map(filePath.wc_str());
 		POST_MESSAGE(SaveMap, (map));
 
-		SetOpenFilename(filename);
+		SetOpenFilename(filePath);
 
 		// Wait for it to finish saving
 		qPing qry;
@@ -943,6 +981,19 @@ void ScenarioEditor::OnSelectedObjectsChange(const std::vector<ObjectID>& select
     GetMenuBar()->Enable(ID_Copy, !selectedObjects.empty());
 }
 
+void ScenarioEditor::OnHelp(wxCommandEvent& event)
+{
+	std::map<int, HelpItem>::const_iterator it = m_HelpData.find(event.GetId());
+	if (it == m_HelpData.end())
+		return;
+	wxMessageDialog* dialog = new wxMessageDialog(
+		nullptr, _("Do you want to open '" + it->second.m_URL + "'?"),
+		_("Atlas"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION
+	);
+	if (dialog->ShowModal() == wxID_YES)
+		wxLaunchDefaultBrowser(it->second.m_URL);
+}
+
 void ScenarioEditor::OnMenuOpen(wxMenuEvent& event)
 {
     // This could be done far more elegantly if wxMenuItem had changeable id.
